import time
from opperai import Opper

opper = Opper()

fn_name = f"docs-snippet-{int(time.time())}"

# --- docs ---
result = opper.call(
    fn_name,
    input={"text": "Summarize this"},
    output_schema={"type": "object", "properties": {"summary": {"type": "string"}}},
)
print(result.data)
# --- /docs ---

# Cleanup
opper.functions.delete(fn_name)
